/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { TransactionURI } from 'symbol-uri-scheme';
import { PublicAccount, MosaicInfo, MosaicId } from 'symbol-sdk';
import { MnemonicPassPhrase, Wallet } from 'symbol-hd-wallets';
import { AllowanceResult, Command, CommandOption, NetworkConfig, Notification, NotificationProof, Standard, TokenIdentifier, TokenPartition, TokenSource, Context, TransactionParameters } from '../../index';
import { SecuritiesMetadata } from './NIP13/models/SecuritiesMetadata';
import { SecuritiesRestrictions } from './NIP13/models/SecuritiesRestrictions';
import { Accountable } from './NIP13/contracts/Accountable';
/**
 * @type NIP13.CommandFn
 * @package standards
 * @since v0.1.0
 * @internal
 * @description Type that describes NIP13 token command functions
 */
declare type CommandFn = (c: Context, i: TokenIdentifier, k: Wallet) => Command;
/**
 * @type NIP13.CommandsList
 * @package standards
 * @since v0.1.0
 * @internal
 * @description Type that describes NIP13 token command lists
 */
declare type CommandsList = {
    [id: string]: CommandFn;
};
/**
 * @type NIP13.TokenMetadata
 * @package standards
 * @since v0.3.3
 * @description Class that describes NIP13 token metadata
 * @see {SecuritiesMetadata}
 */
export declare class TokenMetadata extends SecuritiesMetadata {
}
/**
 * @type NIP13.TokenRestrictions
 * @package standards
 * @since v0.5.1
 * @description Class that describes NIP13 token restrictions
 * @see {SecuritiesRestrictions}
 */
export declare class TokenRestrictions extends SecuritiesRestrictions {
}
/**
 * @type NIP13.TokenAuthority
 * @package standards
 * @since v0.3.3
 * @description Class that describes NIP13 token authorities
 */
export declare class TokenAuthority extends Accountable {
    /**
     * @description The network configuration
     */
    readonly network: NetworkConfig;
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    protected readonly bip39: MnemonicPassPhrase;
    /**
     * @description The deterministic public account representing the authority.
     */
    publicAccount: PublicAccount;
    /**
     * Constructs a NIP13 token authority object.
     *
     * @param {NetworkConfig}       network
     * @param {MnemonicPassPhrase}  bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network: NetworkConfig, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39: MnemonicPassPhrase);
    /**
     * List a token authority's security tokens. A token authority
     * is used to prove the existance of said security tokens on
     * a network.
     *
     * @param {TokenAuthority} authority
     * @return {MosaicId[]}
     */
    static getTokens(authority: PublicAccount | TokenAuthority, network: NetworkConfig): Promise<MosaicId[]>;
}
/**
 * @var NIP13.TokenCommands
 * @package standards
 * @since v0.1.0
 * @description Object that describes NIP13 available token commands
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md#token-commands
 */
export declare const TokenCommands: CommandsList;
/**
 * @var NIP13.Revision
 * @package standards
 * @since v0.1.0
 * @description Object that describes the count of NIP13 security token revisions
 */
export declare const Revision: number;
/**
 * @class NIP13.TokenStandard
 * @package standards
 * @since v0.1.0
 * @description Class that describes NIP13 security tokens standard
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md
 */
export declare class TokenStandard extends Accountable implements Standard {
    /**
     * @description The network configuration
     */
    readonly network: NetworkConfig;
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    protected readonly bip39: MnemonicPassPhrase;
    /**
     * @description The deterministic public account representing the token.
     */
    target: PublicAccount;
    /**
     * @description The token source (network generation hash).
     */
    source: TokenSource;
    /**
     * @description Mosaic information (read from network).
     */
    mosaicInfo: MosaicInfo | undefined;
    /**
     * @description List of operators of said token.
     */
    operators: PublicAccount[];
    /**
     * @description Partition records of said token.
     */
    partitions: TokenPartition[];
    /**
     * @description The securities metadata
     */
    metadata: SecuritiesMetadata | undefined;
    /**
     * @description The securities metadata
     */
    restrictions: SecuritiesRestrictions | undefined;
    /**
     * @description Last token command execution result. URIs must be executed
     *              outside of the token standard.
     */
    result?: TransactionURI;
    /**
     * Constructs a NIP13 token standard object.
     *
     * @param {NetworkConfig}       network
     * @param {MnemonicPassPhrase}  bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network: NetworkConfig, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39: MnemonicPassPhrase);
    /**
     * Getter for the deterministic token identifier.
     *
     * @return {TokenIdentifier}
     */
    get identifier(): TokenIdentifier;
    /**
     * Synchronize the command execution with the network. This method shall
     * be used to fetch data required for execution.
     *
     * @async
     * @return {Promise<boolean>}
     */
    synchronize(): Promise<boolean>;
    /**
     * Creates a new NIP13 Token and configures operators.
     *
     * @param   {string}                  name
     * @param   {PublicAccount}           actor
     * @param   {PublicAccount}           authority
     * @param   {Array<PublicAccount>}    operators
     * @param   {number}                  supply
     * @param   {SecuritiesMetadata}      metadata
     * @param   {TransactionParameters}   parameters
     * @return  {TokenIdentifier}
     **/
    create(name: string, actor: PublicAccount, authority: PublicAccount, operators: PublicAccount[], supply: number, metadata: SecuritiesMetadata, parameters: TransactionParameters): TokenIdentifier;
    /**
     * Add a token holder partition for \a holder with deterministic
     * \a partition account.
     *
     * @param {PublicAccount} actor
     * @param partition
     * @param holder
     */
    addPartition(actor: PublicAccount, partition: PublicAccount, holder: PublicAccount, name: string, parameters: TransactionParameters): Promise<TransactionURI>;
    /**
     * Transfer shares of a Security Token with identifier `tokenId`.
     *
     * @internal This method MUST use the `TransferOwnership` command.
     * @param   {PublicAccount}         actor
     * @param   {PublicAccount}         sender
     * @param   {PublicAccount}         recipient   MUST be a partition account
     * @param   {number}                amount
     * @param   {TransactionParameters} parameters
     * @return  {TransactionURI}
     **/
    transfer(actor: PublicAccount, sender: PublicAccount, recipient: PublicAccount, amount: number, parameters: TransactionParameters): Promise<TransactionURI>;
    /**
     * Transfer shares of a Security Token with identifier `tokenId` and attach
     * plain text-, signed-, encrypted- data.
     *
     * @internal This method MUST use the `TransferOwnershipWithData` command.
     * @param   {PublicAccount}         actor
     * @param   {PublicAccount}         sender
     * @param   {PublicAccount}         recipient   MUST be a partition account
     * @param   {number}                amount
     * @param   {string}                data
     * @param   {TransactionParameters} parameters
     * @return  {TransactionURI}
     **/
    transferWithData(actor: PublicAccount, sender: PublicAccount, recipient: PublicAccount, amount: number, data: string, parameters: TransactionParameters): Promise<TransactionURI>;
    /**
     * Notify an account `account` about `notification`
     *
     * @param   {TokenIdentifier} tokenId
     * @param   {PublicAccount}   account
     * @param   {Notification}    notification
     * @return  {NotificationProof}
     **/
    notify(tokenId: TokenIdentifier, account: PublicAccount, notification: Notification, parameters: TransactionParameters): NotificationProof;
    /**
     * Verifies **allowance** of `sender` to transfer `tokenId` security token
     * to `recipient` with a number of shares attached of `amount`.
     *
     * @param   {TokenIdentifier} tokenId
     * @param   {PublicAccount}   sender
     * @param   {PublicAccount}   recipient
     * @param   {number}          amount
     * @return  {AllowanceResult}
     **/
    canTransfer(tokenId: TokenIdentifier, sender: PublicAccount, recipient: PublicAccount, amount: number): AllowanceResult;
    /**
     * Verifies **allowance** of `sender` to transfer `tokenId` security token
     * to `recipient` with a number of shares attached of `amount` and attaching
     * `data` plain text-, signed- or encrypted- data.
     *
     * @param   {TokenIdentifier} tokenId
     * @param   {PublicAccount}   sender
     * @param   {PublicAccount}   recipient
     * @param   {number}          amount
     * @param   {string}          data
     * @return  {AllowanceResult}
     **/
    canTransferWithData(tokenId: TokenIdentifier, sender: PublicAccount, recipient: PublicAccount, amount: number, data: string): AllowanceResult;
    /**
     * Verifies **allowance** of `operator` to execute `command` with `tokenId` security token.
     *
     * @internal This method MUST use the `Command.canExecute()` method.
     * @param   {PublicAccount}         actor
     * @param   {TokenIdentifier}       tokenId
     * @param   {string}                command
     * @param   {Array<CommandOption>}  argv
     * @return  {AllowanceResult}
     **/
    canExecute(actor: PublicAccount, tokenId: TokenIdentifier, command: string, argv: CommandOption[]): AllowanceResult;
    /**
     * Execute `command` for Security Token with identifier `tokenId`. Arguments
     * the command execution can be passed in `argv`.
     *
     * This method MUST call the `synchronize()` method.
     *
     * @internal This method MUST use the `Command.execute()` method.
     * @param   {PublicAccount}         actor
     * @param   {TokenIdentifier}       tokenId
     * @param   {string}                command
     * @param   {TransactionParameters} parameters
     * @param   {Array<CommandOption>}  argv
     * @return  {Promise<TransactionURI>}
     **/
    execute(actor: PublicAccount, tokenId: TokenIdentifier, command: string, parameters: TransactionParameters, argv: CommandOption[]): Promise<TransactionURI>;
    /**
     * Execute `command` for Security Token with identifier `tokenId`. Arguments
     * the command execution can be passed in `argv`.
     *
     * This method MUST NOT call the `synchronize()` method.
     *
     * @internal This method MUST use the `Command.execute()` method.
     * @param   {PublicAccount}         actor
     * @param   {TokenIdentifier}       tokenId
     * @param   {string}                command
     * @param   {TransactionParameters} parameters
     * @param   {Array<CommandOption>}  argv
     * @return  {TransactionURI}
     **/
    executeWithoutSync(actor: PublicAccount, tokenId: TokenIdentifier, command: string, parameters: TransactionParameters, argv: CommandOption[]): TransactionURI;
    /**
     * Gets an execution context
     *
     * @param   {PublicAccount}           actor
     * @param   {TransactionParameters}   parameters
     * @param   {CommandOption[]}         argv
     * @return  {Context}
     */
    protected getContext(actor: PublicAccount, parameters: TransactionParameters, argv?: CommandOption[]): Context;
    /**
     * Gets a command instance around `context` and `tokenId`.
     *
     * @param {TokenIdentifier} tokenId
     * @param {string}          command
     * @param {Context}         context
     * @return {Command}
     */
    protected getCommand(tokenId: TokenIdentifier, command: string, context: Context): Command;
}
/**
 * @class NIP13.Token
 * @package standards
 * @since v0.1.0
 * @description Class that describes NIP13 compliant security tokens
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md
 */
export declare class Token extends TokenStandard {
}
export {};
