"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const AbstractCommand_1 = require("./AbstractCommand");
/**
 * @class NIP13.CreatePartition
 * @package NIP13 Token Commands
 * @since v0.5.1
 * @description Class that describes a token command for creating token holder partitions for NIP13 compliant tokens.
 * @summary
 * This token command accepts the following arguments:
 *
 * | Argument | Description | Example |
 * | --- | --- | --- |
 * | name | Name of the token holder partition | `"NIP13 Example"` |
 * | partition | Token holder partition account | `new PublicAccount(...)` |
 * | holder | Actual token holder public account | `new PublicAccount(...)` |
 */
class CreatePartition extends AbstractCommand_1.AbstractCommand {
    constructor() {
        super(...arguments);
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [
            'name',
            'partition',
            'holder',
        ];
        // end-region abstract methods
    }
    // region abstract methods
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name() {
        return 'CreatePartition';
    }
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor() {
        return 'NIP13(v' + this.context.revision + ')' + ':partition:' + this.identifier.id;
    }
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `CreatePartition` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    get transactions() {
        // read external arguments
        const name = this.context.getInput('name', 'default');
        const partition = this.context.getInput('partition', new symbol_sdk_1.PublicAccount());
        const holder = this.context.getInput('holder', new symbol_sdk_1.PublicAccount());
        // find partition
        const the_partition = this.partitions.find(p => p.account.address.equals(partition.address));
        if (undefined !== the_partition) {
            //XXX partition already exists
            return [];
        }
        // prepare output
        const transactions = [];
        const signers = [];
        // Transaction 01: MultisigAccountModificationTransaction
        // :warning: Token holder is made optional in both minApproval and minRemoval.
        transactions.push(symbol_sdk_1.MultisigAccountModificationTransaction.create(this.context.parameters.deadline, this.operators.length, // all operators for minApproval (holder is optional)
        this.operators.length, // all except one for minRemoval (holder is optional)
        this.operators // operators
            .concat([holder]), // + token holder
        [], this.context.network.networkType, undefined));
        // Transaction 01 is issued by **partition** account
        signers.push(partition);
        // Transaction 02: AccountMetadataTransaction
        transactions.push(symbol_sdk_1.AccountMetadataTransaction.create(this.context.parameters.deadline, partition.publicKey, symbol_sdk_1.KeyGenerator.generateUInt64Key('NAME'), name.length, name, this.context.network.networkType, undefined));
        // Transaction 02 is issued by **partition** account
        signers.push(partition);
        // Transaction 03: AccountMetadataTransaction
        transactions.push(symbol_sdk_1.AccountMetadataTransaction.create(this.context.parameters.deadline, partition.publicKey, symbol_sdk_1.KeyGenerator.generateUInt64Key('OWNER'), holder.address.plain().length, holder.address.plain(), this.context.network.networkType, undefined));
        // Transaction 03 is issued by **partition** account
        signers.push(partition);
        // Transaction 03: AccountMosaicRestrictionTransaction
        // :note: This transaction authorizes mosaicId and networkCurrencyMosaicId for partition
        transactions.push(symbol_sdk_1.AccountMosaicRestrictionTransaction.create(this.context.parameters.deadline, symbol_sdk_1.AccountRestrictionFlags.AllowMosaic, [this.identifier.toMosaicId(), this.context.network.feeMosaicId], // MosaicId & networkCurrencyMosaicId
        [], this.context.network.networkType, undefined));
        // Transaction 03 is issued by **target** account (multisig)
        signers.push(partition);
        // Transaction 04: MosaicAddressRestriction for target address
        // :note: This transaction authorizes the partition account by adding a User_Role=2 (Holder)
        transactions.push(symbol_sdk_1.MosaicAddressRestrictionTransaction.create(this.context.parameters.deadline, this.identifier.toMosaicId(), symbol_sdk_1.KeyGenerator.generateUInt64Key('User_Role'), partition.address, symbol_sdk_1.UInt64.fromUint(2), // newRestrictionValue: 2 = Holder
        this.context.network.networkType, undefined, // previousRestrictionValue
        undefined));
        // Transaction 04 is issued by the target account
        signers.push(this.target);
        // Transaction 05: Add execution proof transaction
        transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, partition.address, [], symbol_sdk_1.PlainMessage.create(this.descriptor + ':' + name), this.context.network.networkType, undefined));
        // Transaction 1.05 is issued by **target** account
        signers.push(this.target);
        // return transactions issued by assigned signer
        return transactions.map((transaction, i) => transaction.toAggregate(signers[i]));
    }
}
exports.CreatePartition = CreatePartition;
