/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { PublicAccount, Transaction } from 'symbol-sdk';
import { CommandOption, AllowanceResult } from '../../../../index';
import { AbstractCommand } from './AbstractCommand';
/**
 * @class NIP13.CreateToken
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Class that describes a token command for creating NIP13 compliant tokens.
 * @summary
 * This token command accepts the following arguments:
 *
 * | Argument | Description | Example |
 * | --- | --- | --- |
 * | name | Name of the security token | `"NIP13 Example"` |
 * | source | Source network | `"Symbol Mainnet"` |
 * | authority | Token Authority | `new PublicAccount(...)` |
 * | operators | Security Token Operators | `[new PublicAccount(...)]` |
 * | supply | Total number of outstanding shares | `123456` |
 * | metadata | Metadata associated at token level | `{'MIC': 'XNAS', ...}` |
 */
export declare class CreateToken extends AbstractCommand {
    /**
     * @description List of **required** arguments for this token command.
     * @link https://en.wikipedia.org/wiki/International_Securities_Identification_Number
     * @link https://en.wikipedia.org/wiki/ISO_10962
     * @link https://en.wikipedia.org/wiki/Market_Identifier_Code
     */
    arguments: string[];
    /**
     * Synchronize the command execution with the network. This method shall
     * be used to fetch data required for execution.
     *
     * @async
     * @override The 'CreateToken' command does not need synchronization.
     * @return {Promise<boolean>}
     */
    synchronize(): Promise<boolean>;
    /**
     * @description Method that verifies the allowance of an operator to
     *              execute the token command `CreateToken`.
     * @see {BaseCommand.canExecute}
     **/
    canExecute(actor: PublicAccount, argv?: CommandOption[]): AllowanceResult;
    /**
     * @description Getter for the command name.
     * @see {AbstractCommand.name}
     **/
    get name(): string;
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor(): string;
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `CreateToken` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    protected get transactions(): Transaction[];
}
