"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const AbstractCommand_1 = require("./AbstractCommand");
/**
 * @class NIP13.TransferOwnership
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Class that describes a token command for transferring NIP13 compliant tokens.
 * @summary
 * This token command accepts the following arguments:
 *
 * | Argument | Description | Example |
 * | --- | --- | --- |
 * | sender | Sender token holder partition account (or use target account) | `new PublicAccount(...)` |
 * | recipient | Recipient token holder partition account | `new PublicAccount(...)` |
 * | amount | Number of shares to be transferred | `1` |
 */
class TransferOwnership extends AbstractCommand_1.AbstractCommand {
    constructor() {
        super(...arguments);
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [
            'sender',
            'recipient',
            'amount',
        ];
        // end-region abstract methods
    }
    // region abstract methods
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name() {
        return 'TransferOwnership';
    }
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor() {
        return 'NIP13(v' + this.context.revision + ')' + ':transfer:' + this.identifier.id;
    }
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `TransferOwnership` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    get transactions() {
        // read external arguments
        const sender = this.context.getInput('sender', new symbol_sdk_1.PublicAccount());
        const recipient = this.context.getInput('recipient', new symbol_sdk_1.PublicAccount());
        const amount = this.context.getInput('amount', 0);
        // find recipient partition
        const the_partition = this.partitions.find(p => p.account.address.equals(recipient.address));
        if (undefined === the_partition) {
            // Error: partition does not exist
            return [];
        }
        // prepare output
        const transactions = [];
        const signers = [];
        // 1) sending from **target** account
        if (sender.address.equals(this.target.address)) {
            // Transaction 1.01: Add ownership transfer transaction
            transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, the_partition.account.address, [
                new symbol_sdk_1.Mosaic(this.identifier.toMosaicId(), symbol_sdk_1.UInt64.fromUint(amount))
            ], // mosaics will be owned by partition account
            symbol_sdk_1.PlainMessage.create(this.descriptor + ':' + the_partition.name), this.context.network.networkType, undefined));
            // Transaction 1.01 is issued by **sender** account
            signers.push(sender);
        }
        // 2) sending from **partition** account (not target account)
        //    should change ownership from **owner** to **recipient**
        else {
            // find sender partition
            const sender_partition = this.partitions.find((p) => p.account.address.equals(sender.address));
            if (sender_partition === undefined) {
                // Error: sender partition does not exist
                return [];
            }
            // Transaction 2.01: First send back the amount to the target account
            transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, this.target.address, // back to target account (non-transferrable)
            [
                new symbol_sdk_1.Mosaic(this.identifier.toMosaicId(), symbol_sdk_1.UInt64.fromUint(amount))
            ], symbol_sdk_1.EmptyMessage, // This transaction does not need a marker
            this.context.network.networkType, undefined));
            // Transaction 2.01 is issued by **sender** account
            signers.push(sender);
            // Transaction 2.02: Add ownership transfer transaction
            transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, the_partition.account.address, // new recipient is the recipient partition account
            [
                new symbol_sdk_1.Mosaic(this.identifier.toMosaicId(), symbol_sdk_1.UInt64.fromUint(amount))
            ], symbol_sdk_1.PlainMessage.create(this.descriptor + ':' + the_partition.name), // use recipient partition name
            this.context.network.networkType, undefined));
            // Transaction 2.02 is issued by **target** account
            signers.push(this.target);
        }
        // return transactions issued by assigned signer
        return transactions.map((transaction, i) => transaction.toAggregate(signers[i]));
    }
}
exports.TransferOwnership = TransferOwnership;
