/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { Account, PublicAccount } from 'symbol-sdk';
import { MnemonicPassPhrase, Wallet } from 'symbol-hd-wallets';
import { NetworkConfig } from '../../../../index';
/**
 * @class Accountable
 * @package NIP13/contracts
 * @since v0.3.0
 * @description Class that describes an object able to create accounts.
 */
export declare class Accountable {
    /**
     * @description The network configuration
     */
    protected readonly network: NetworkConfig;
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    protected readonly bip39: MnemonicPassPhrase;
    /**
     * @description The key provider (Wallet instanciated through BIP32 extended key).
     */
    protected keyProvider: Wallet;
    /**
     * Construct an `Accountable` object around \a bip39 mnemonic pass phrase.
     *
     * @param {MnemonicPassPhrase} bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network: NetworkConfig, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39: MnemonicPassPhrase);
    /**
     * Derive a child account
     *
     * @param {string}    derivationPath
     * @return {Account}
     */
    protected getAccount(derivationPath: string): Account;
    /**
     * Derive an **authority** account
     *
     * @return {Account}
     */
    getAuthority(): Account;
    /**
     * Derive a **target** account
     *
     * @return {Account}
     */
    getTarget(): Account;
    /**
     * Derive an **operator** account
     *
     * @internal Operator account derivation paths use the REMOTE (BIP44=CHANGE) path level.
     * @param {number}  at
     * @return {Account}
     */
    getOperator(at?: number): Account;
    /**
     * Derive a **partition** account
     *
     * @internal Partition account derivation paths use the ADDRESS path level.
     * @param   {PublicAccount}  owner
     * @param   {number}         at
     * @param   {string}         name
     * @return  {Account}
     */
    getPartition(owner: PublicAccount, name?: string): Account;
    /**
     * Get the BIP44 path for a partition owned by `owner`
     *
     * @internal Partition account derivation paths use the ADDRESS path level.
     * @param   {PublicAccount} owner
     * @param   {number}         at
     * @return  {string}
     */
    getPathForPartition(owner: PublicAccount, name?: string): string;
}
