"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const index_1 = require("../../../../index");
const SecuritiesMetadata_1 = require("../models/SecuritiesMetadata");
/**
 * @class MetadataService
 * @package services
 * @since v0.1.0
 * @description Class that describes a service around token metadata features.
 */
class MetadataService extends index_1.Service {
    /**
     * Helper function to format metadata entries
     *
     * @param {Metadata} metadataEntry
     * @return {any}
     */
    formatMetadata(metadata) {
        const metadataEntry = metadata.metadataEntry;
        return (Object.assign(Object.assign({}, metadataEntry), { scopedMetadataKey: metadataEntry.scopedMetadataKey.toHex(), senderAddress: symbol_sdk_1.Address.createFromPublicKey(metadataEntry.senderPublicKey, this.context.network.networkType), targetAddress: symbol_sdk_1.Address.createFromPublicKey(metadataEntry.targetPublicKey, this.context.network.networkType), metadataType: symbol_sdk_1.MetadataType.Mosaic, targetId: metadataEntry.targetId ? metadataEntry.targetId.toHex() : undefined, metadataValue: metadataEntry.value }));
    }
    /**
     * Read token partitions from token operators
     * network information.
     *
     * @param {TokenIdentifier}       tokenId     The token identifier.
     * @return {SecuritiesMetadata}
     */
    getMetadataFromNetwork(tokenId) {
        return __awaiter(this, void 0, void 0, function* () {
            // read metadata entries for token from network
            const entries = yield this.context.network.factoryHttp.createMetadataRepository()
                .getMosaicMetadata(tokenId.toMosaicId(), new symbol_sdk_1.QueryParams({ pageSize: 10, id: undefined }))
                .toPromise();
            const mosaicMetadata = entries.map((metadata) => this.formatMetadata(metadata));
            // overwrite `scopeMetadataKey` values to rewrite keys
            const metadataFields = Object.keys(MetadataService.KNOWN_METADATAS);
            mosaicMetadata.sort((a, b) => {
                const indexA = metadataFields.findIndex(v => v === a.scopedMetadataKey);
                const indexB = metadataFields.findIndex(v => v === b.scopedMetadataKey);
                return indexA - indexB;
            });
            // read only known metadata
            let known = {};
            for (let i = 0, m = metadataFields.length; i < m; i++) {
                const entry = mosaicMetadata[i];
                if (metadataFields.indexOf(entry.scopedMetadataKey) !== -1) {
                    const key = MetadataService.getKnownMetadataKey(entry.scopedMetadataKey);
                    known[key] = entry.metadataValue;
                }
            }
            return new SecuritiesMetadata_1.SecuritiesMetadata(known['nip13_mic'], known['nip13_isin'], known['nip13_iso10962'], known['nip13_website'], known['nip13_sector'], known['nip13_industry'], {
                'nip13_token_identifier': known['nip13_token_identifier'],
                'nip13_name': known['nip13_name'],
            });
        });
    }
}
exports.MetadataService = MetadataService;
/**
 * @description Known NIP13 metadata keys
 */
MetadataService.KNOWN_METADATAS = {
    'D434152406E75CA0': 'nip13_token_identifier',
    '8B5DD479E6AB718A': 'nip13_name',
    'CCDCF598A830B275': 'nip13_sector',
    'B5C804D03E2CC11F': 'nip13_industry',
    'BC2FC3ACFF58FF89': 'nip13_isin',
    'D92F12883E1687AA': 'nip13_mic',
    '9E600698F53ED4F8': 'nip13_iso10962',
    '985A5AFB4D783C53': 'nip13_website'
};
/**
 * Helper function to retrieve known mosaic metadata keys
 *
 * @param {string}  hexKey
 * @return {string}
 */
MetadataService.getKnownMetadataKey = (hexKey) => {
    return hexKey in MetadataService.KNOWN_METADATAS
        ? MetadataService.KNOWN_METADATAS[hexKey]
        : hexKey;
};
