"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// internal dependencies
const index_1 = require("../../../../index");
const TransactionService_1 = require("./TransactionService");
const MultisigService_1 = require("./MultisigService");
/**
 * @class PartitionService
 * @package services
 * @since v0.1.0
 * @description Class that describes a service around token partition features.
 */
class PartitionService extends index_1.Service {
    /**
     * Read token partitions from token operators
     * network information.
     *
     * @param {TokenIdentifier}       tokenId     The token identifier.
     * @param {PublicAccount}         target      The deterministic account that represents the token.
     * @param {PublicAccount[]}       operators   The list of operators of said token.
     * @param {string}                descriptor  The token command descriptor used as a marker.
     * @return {TokenPartition[]}
     */
    getPartitionsFromNetwork(tokenId, target, operators, descriptor = '') {
        return __awaiter(this, void 0, void 0, function* () {
            // initialize APIs
            const accountHttp = this.context.network.factoryHttp.createAccountRepository();
            const multisigHttp = this.context.network.factoryHttp.createMultisigRepository();
            const multisig = new MultisigService_1.MultisigService(this.context);
            const service = new TransactionService_1.TransactionService(accountHttp, this.context.network.factoryHttp.createChainRepository(), this.context.network.factoryHttp.createTransactionRepository(), this.context.network.factoryHttp.createReceiptRepository(), 100);
            // prepare output
            const partitions = [];
            // Step 1) read transfers to find partition accounts
            const transactions = yield service.getOutgoingTransfers(target.address, undefined, 0).toPromise();
            // filter partition transfer transactions
            const markedTransfers = transactions.filter(tx => tx.message.payload.startsWith(descriptor));
            // read recipients to find partition accounts
            const addresses = markedTransfers.map(t => t.recipientAddress).filter(addr => !addr.equals(target.address));
            // Step 2) read partition accounts information from network
            const infos = yield accountHttp.getAccountsInfo(addresses).toPromise();
            // Step 3) iterate partition accounts
            for (let i = 0, m = infos.length; i < m; i++) {
                const accountInfo = infos[i];
                // Step 3.1) get partition account multisig graph
                const graph = yield multisigHttp.getMultisigAccountGraphInfo(accountInfo.address).toPromise();
                const owner = multisig.getMultisigAccountInfoFromGraph(graph).map((cosig) => cosig.cosignatories).reduce((prev, it) => {
                    const ops = operators.map(o => o.address.plain());
                    return prev.concat(it.filter(c => !ops.includes(c.address.plain())));
                }, [])[0]; // force one
                // Step 3.2) fetch partition account incoming transaction
                const lastMarker = markedTransfers.filter(tx => tx.recipientAddress.equals(accountInfo.address)).shift();
                if (lastMarker === undefined) {
                    // invalid partition configuration
                    continue;
                }
                // Step 3.3) read payload to identify partition label
                const payload = lastMarker.message.payload;
                const label = payload.substr(descriptor.length); // take everything to the right of the descriptor
                // amount is either of 0 or partition amount
                let partitionAmount = 0;
                if (accountInfo.mosaics.length) {
                    // Step 3.4) read partition amount
                    const tokenEntry = accountInfo.mosaics.filter(mosaic => mosaic.id.equals(tokenId.toMosaicId())).shift();
                    if (tokenEntry !== undefined) {
                        partitionAmount = tokenEntry.amount.compact();
                    }
                }
                // Step 3.5) register partition
                partitions.push(new index_1.TokenPartition(label, owner, accountInfo.publicAccount, partitionAmount));
            }
            return partitions;
        });
    }
}
exports.PartitionService = PartitionService;
