import { Observable } from 'rxjs';
import { AccountRepository, Address, ChainRepository, MosaicId, ReceiptRepository, Transaction, TransactionRepository, TransferTransaction } from 'symbol-sdk';
export declare class TransactionService {
    private readonly accountRepository;
    private readonly chainRepository;
    private readonly transactionHttp;
    private readonly receiptHttp;
    private pageSize;
    private readonly transactionService;
    constructor(accountRepository: AccountRepository, chainRepository: ChainRepository, transactionHttp: TransactionRepository, receiptHttp: ReceiptRepository, pageSize: number);
    /**
     * Processes the list of transactions. In case there is an aggregate transaction,
     * removes the envelop returning separately each inner transaction.
     *
     * @param transactions List of transactions.
     * @returns Transaction[]
     */
    static flattenAggregateTransactions(transactions: Transaction[]): Transaction[];
    /**
     * Given an array of transactions, returns the ones with type TRANSFER.
     *
     * @param transactions List of transactions.
     * @returns TransferTransaction[]
     */
    static filterTransferTransactions(transactions: Transaction[]): TransferTransaction[];
    /**
     * Gets the transaction hash.
     * If it is an aggregate transaction, returns an aggregate transaction hash.
     *
     * @param transaction Transaction.
     * @returns string | undefined
     */
    static getTransactionHash(transaction: Transaction): string | undefined;
    /**
     * Given an array of transactions, returns the ones that are eligible to be _transfers_.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible _transfers_.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId (Optional) Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleIncomingTransfers(transactions: Transaction[], exchangeAddress: Address, tokenId: MosaicId | undefined): TransferTransaction[];
    /**
     * Given an array of transactions, returns the ones that are eligible to be _transfers_.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible _transfers_.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId (Optional) Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleOutgoingTransfers(transactions: Transaction[], exchangeAddress: Address, tokenId: MosaicId | undefined): TransferTransaction[];
    /**
     * Given an array of transactions, returns the ones that are eligible to be deposits.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible deposits.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleDeposits(transactions: Transaction[], exchangeAddress: Address, tokenId: MosaicId): TransferTransaction[];
    /**
     * Given an array of transactions, returns the ones that are eligible to be deposits.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The signer public key is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible deposits.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleWithdrawals(transactions: Transaction[], exchangeAddress: Address, tokenId: MosaicId): TransferTransaction[];
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getIncomingTransfers(exchangeAddress: Address, tokenId: MosaicId | undefined, requiredConfirmations: number, lastTransactionId?: string): Observable<TransferTransaction[]>;
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getOutgoingTransfers(exchangeAddress: Address, tokenId: MosaicId | undefined, requiredConfirmations: number, lastTransactionId?: string): Observable<TransferTransaction[]>;
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getDeposits(exchangeAddress: Address, tokenId: MosaicId, requiredConfirmations: number, lastTransactionId?: string): Observable<TransferTransaction[]>;
    /**
     * Gets the transactions eligible to be withdrawals pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getWithdrawals(exchangeAddress: Address, tokenId: MosaicId, requiredConfirmations: number, lastTransactionId?: string): Observable<TransferTransaction[]>;
    /**
     * Gets the list of transactions pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <Transaction[]>
     */
    getUnprocessedTransactions(exchangeAddress: Address, lastTransactionId?: string): Observable<Transaction[]>;
    /**
     * Gets deposits with enough confirmations.
     * @param transactions Array of transactions to check.
     * @param requiredConfirmations Number of confirmations to consider a transaction valid.
     * @returns Observable <Transaction[]>
     */
    filterTransactionsWithEnoughConfirmations(transactions: Transaction[], requiredConfirmations: number): Observable<Transaction[]>;
    /**
     * Resolves the aliases behind an announced transaction.
     * @param transactions Array of transactions to resolve.
     * @returns Observable <Transaction[]>
     */
    resolveTransactionsAliases(transactions: Transaction[]): Observable<Transaction[]>;
}
